@@ -66,6 +66,9 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
     expressionMetabolicS = expressionMetabolicS.apply(pd.to_numeric, errors='coerce')
     
     
+    fRange = [i for i in fRange if i<=numSamples]
+    
+    
 # =============================================================================
 # Connectivity    
 # =============================================================================
@@ -108,6 +111,8 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
         numNodEdgBinP = np.where(numNodEdgP<minNumNodes,0,1)
         connAnalysisPatients = connAnalysisPatientsAll[0]*numNodEdgBinP
         
+        connAnalysisDiff = connAnalysisPatients-connAnalysisControl
+        
         #revise random connectivity wrt min num of nodes
         randomConnectivityControlMeanB = randomConnectivityControlMean * numNodEdgBinC
         randomConnectivityControlStdB = randomConnectivityControlStd * numNodEdgBinC
@@ -125,33 +130,125 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
      
     
         # find the index that max diff
-        indexMax = list(zScoreDiff.stack().idxmax())
+        indexMaxZ = list(zScoreDiff.stack().idxmax())
         #maxZscore = zScoreDiff.max().max()
        
     
         ##### Max Diff Filtered Genes #####
-        pG, f = indexMax
+        pGZ, fZ = indexMaxZ
 
-        numUpDown = math.ceil(len(expressionMetabolicS) * pG)
+        numUpDownZ = math.ceil(len(expressionMetabolicS) * pGZ)
+        
+        
+        PatientGenesZ_patients = filteredGenesGeneBase(GCS, Patients, expressionMetabolicS,
+                                                numUpDownZ, fZ, upORdownRegulated)
+        
+        PatientGenesZ_controls = filteredGenesGeneBase(GCS, Controls, expressionMetabolicS,
+                                                numUpDownZ, fZ, upORdownRegulated)
         
-        PatientGenes = filteredGenesGeneBase(GCS, Patients, expressionMetabolicS,
-                                                numUpDown, f, upORdownRegulated)
         #connectivity
-        SP = GCS.subgraph(PatientGenes)        
+        SPZ = GCS.subgraph(PatientGenesZ_patients)        
         
         
         ##### GWAS Match #####
-        Intersection = len(set(SP.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
-        
+        IntersectionZ = len(set(SPZ.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+        detectedDiseaseGenesZ = set(SPZ.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl)))
         diseaseGenesInGraph = len(set(GCS.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
     
+        
+# =============================================================================
+# Intersection vs Random    
+# =============================================================================
+        if SPZ.number_of_nodes() == GCS.number_of_nodes():
+                        
+            intRandomMeanZ = IntersectionZ#####################
+            intRandomStdZ = 0
+            intZscoreZ = 0
+            
+        elif SPZ.number_of_nodes() == 0:
+            intRandomMeanZ = 0
+            intRandomStdZ = 0
+            intZscoreZ = 0       
+            
+        else:
+            numDetGeneZ = SPZ.number_of_nodes()
+            
+            intersectionSampleSizeZ=[]
+            for i in range(numRandomGroups):
+                sampZ = random.sample(list(expressionMetabolicS.columns), numDetGeneZ)
+                intersectionSampleZ = len(set(diseaseGenesListEnsembl.ensembl).intersection(set(sampZ)))
+                intersectionSampleSizeZ.append(intersectionSampleZ)
+                
+        
+            intRandomMeanZ = np.mean(intersectionSampleSizeZ)
+            intRandomStdZ = np.std(intersectionSampleSizeZ)
+            intZscoreZ = (IntersectionZ-intRandomMeanZ)/intRandomStdZ########################
+       
+# =============================================================================
+#     
+# =============================================================================
+        
+        # find the index that max diff
+        indexMaxC = list(connAnalysisDiff.stack().idxmax())
+        #maxZscore = zScoreDiff.max().max()
+       
     
-        ##### SAVE #####
+        ##### Max Diff Filtered Genes #####
+        pGC, fC = indexMaxC
+
+        numUpDownC = math.ceil(len(expressionMetabolicS) * pGC)
+        
+        
+        PatientGenesC_patients = filteredGenesGeneBase(GCS, Patients, expressionMetabolicS,
+                                                numUpDownC, fC, upORdownRegulated)
+        
+        PatientGenesC_controls = filteredGenesGeneBase(GCS, Controls, expressionMetabolicS,
+                                                numUpDownC, fC, upORdownRegulated)
+        
+        #connectivity
+        SPC = GCS.subgraph(PatientGenesC_patients)        
+        
+        
+        ##### GWAS Match #####
+        IntersectionC = len(set(SPC.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+        detectedDiseaseGenesC = set(SPC.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl)))
+        #diseaseGenesInGraph = len(set(GCS.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+    
+ 
+        if SPC.number_of_nodes() == GCS.number_of_nodes():
+                        
+            intRandomMeanC = IntersectionC#####################
+            intRandomStdC = 0
+            intZscoreC = 0
+            
+        elif SPC.number_of_nodes() == 0:
+            intRandomMeanC = 0
+            intRandomStdC = 0
+            intZscoreC = 0       
+            
+        else:
+            numDetGeneC = SPC.number_of_nodes()
+            
+            intersectionSampleSizeC=[]
+            for i in range(numRandomGroups):
+                sampC = random.sample(list(expressionMetabolicS.columns), numDetGeneC)
+                intersectionSampleC = len(set(diseaseGenesListEnsembl.ensembl).intersection(set(sampC)))
+                intersectionSampleSizeC.append(intersectionSampleC)
+                
+        
+            intRandomMeanC = np.mean(intersectionSampleSizeC)
+            intRandomStdC = np.std(intersectionSampleSizeC)
+            intZscoreC = (IntersectionC-intRandomMeanC)/intRandomStdC########################
+
+# =============================================================================
+#  Save       
+# =============================================================================
         
         resultDict = {"disease":disease, "network": network,
                     "currencyMetaPer": currencyMetaPer,
                     "upORdownRegulated": upORdownRegulated,
                     "method": method,
+                    "combination": combination,
                     "minNumberNodes": minNumNodes, #eliminate graphs that has less than "minNumberNodes" nodes
                     "numRandomGroups":numRandomGroups,#number of randomly generated groups to estimate avg, std connectivity
                     "numNodes":GCS.number_of_nodes(),
@@ -160,28 +257,50 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
                     
                     "numDiseaseGenes":len(diseaseGenesListEnsembl),
                     "numDiseaseGenesInGraph":diseaseGenesInGraph,
-                    "numDetectedDiseaseGenes":SP.number_of_nodes(),
                     
-                    "subgraphEdges":SP.number_of_edges(),
-                    "maxDiffIndex_pG":indexMax[0],
-                    "maxDiffIndex_f":indexMax[1],
                     
-                    "maxDiffPatientConn":connAnalysisPatients.loc[pG, f],
-                    "maxDiffControlConn":connAnalysisControl.loc[pG, f],
-                    "maxDiffConnDiff":connAnalysisPatients.loc[pG, f]-connAnalysisControl.loc[pG, f],
-                    "maxDiffPatientZScore":connAnalysisPatient_zscore.loc[pG, f],
-                    "maxDiffControlZScore":connAnalysisControl_zscore.loc[pG, f],
-                    "maxDiffZScoreDiff":connAnalysisPatient_zscore.loc[pG, f]-connAnalysisControl_zscore.loc[pG, f],
+                    "numDetectedDiseaseGenesBYZ":SPZ.number_of_nodes(),
+                    
+                    "subgraphEdgesBYZ":SPZ.number_of_edges(),
+                    "maxDiffIndex_pG_BYZ":indexMaxZ[0],
+                    "maxDiffIndex_f_BYZ":indexMaxZ[1],
+                    
+                    "maxDiffPatientConn_BYZ":connAnalysisPatients.loc[pGZ, fZ],
+                    "maxDiffControlConn_BYZ":connAnalysisControl.loc[pGZ, fZ],
+                    "maxDiffConnDiff_BYZ":connAnalysisPatients.loc[pGZ, fZ]-connAnalysisControl.loc[pGZ, fZ],
+                    "maxDiffPatientZScore_BYZ":connAnalysisPatient_zscore.loc[pGZ, fZ],
+                    "maxDiffControlZScore_BYZ":connAnalysisControl_zscore.loc[pGZ, fZ],
+                    "maxDiffZScoreDiff_BYZ":connAnalysisPatient_zscore.loc[pGZ, fZ]-connAnalysisControl_zscore.loc[pGZ, fZ],
+                    
+                    "intersection_BYZ":IntersectionZ,
+                    
+                    "randomIntersectionNumNodesMean_BYZ": intRandomMeanZ,
+                    "randomIntersectionNumNodesStd_BYZ": intRandomStdZ,
+                    "randomIntersectionNumNodesZScore_BYZ": intZscoreZ,
+                    
+                    
+                    
+                    "numDetectedDiseaseGenesBYC":SPC.number_of_nodes(),
+                    
+                    "subgraphEdgesBYC":SPC.number_of_edges(),
+                    "maxDiffIndex_pG_BYC":indexMaxC[0],
+                    "maxDiffIndex_f_BYC":indexMaxC[1],
+                    
+                    "maxDiffPatientConn_BYC":connAnalysisPatients.loc[pGC, fC],
+                    "maxDiffControlConn_BYC":connAnalysisControl.loc[pGC, fC],
+                    "maxDiffConnDiff_BYC":connAnalysisPatients.loc[pGC, fC]-connAnalysisControl.loc[pGC, fC],
+                    
+                    
+                    "intersection_BYC":IntersectionC,
+                    
+                    "randomIntersectionNumNodesMean_BYC": intRandomMeanC,
+                    "randomIntersectionNumNodesStd_BYC": intRandomStdC,
+                    "randomIntersectionNumNodesZScore_BYC": intZscoreC
                     
-                    "intersection":Intersection
                     }
         
         resultDF = pd.DataFrame.from_dict(resultDict, orient="index")
         resultDF0 = pd.concat([resultDF0, resultDF.T])
-        
-# =============================================================================
-# Save         
-# =============================================================================
       
         
         fn1="result/" + directory + "/"
@@ -189,7 +308,13 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
         
         resultDF0.to_csv(fn1+"resultSummary"+fn2)
         
-        pd.DataFrame(PatientGenes).to_csv(fn1+"patientGeneList"+fn2)
+        pd.DataFrame(PatientGenesZ_patients).to_csv(fn1+"patientGeneListBYZ"+fn2)
+        pd.DataFrame(PatientGenesZ_controls).to_csv(fn1+"controlGeneListBYZ"+fn2)
+        pd.DataFrame(detectedDiseaseGenesZ).to_csv(fn1+"detectedDiseaseGenesBYZ"+fn2)
+        
+        pd.DataFrame(PatientGenesC_patients).to_csv(fn1+"patientGeneListBYZ"+fn2)
+        pd.DataFrame(PatientGenesC_controls).to_csv(fn1+"controlGeneListBYZ"+fn2)
+        pd.DataFrame(detectedDiseaseGenesC).to_csv(fn1+"detectedDiseaseGenesBYZ"+fn2)
         
         numNodEdgC.to_csv(fn1+"numberNodesControl"+fn2)
         numNodEdgP.to_csv(fn1+"numberNodesPatient"+fn2)
@@ -218,7 +343,10 @@ def geneBased(disease, network, networkPath, currencyMetaPer,
 def filteredGenesGeneBase(G, Patients, expressionMetabolic, numUpDown, f,
                           upORdownRegulated):
     
-    allDiseaseRelated={}
+    
+    
+    expressionMetabolicBinary = pd.DataFrame(index=expressionMetabolic.index,
+                                           columns=expressionMetabolic.columns, data=0)
 
     for i in list(expressionMetabolic.columns):
         if upORdownRegulated == "updown":
@@ -236,14 +364,14 @@ def filteredGenesGeneBase(G, Patients, expressionMetabolic, numUpDown, f,
             z = pd.Series(z)
             patientsList = list(z.sort_values(ascending=False).tail(numUpDown).index)      
         
-        expressionMetabolic.loc[:,i] = 0
-        expressionMetabolic.loc[patientsList,i] =1 
+        #expressionMetabolic.loc[:,i] = 0
+        expressionMetabolicBinary.loc[patientsList,i] =1 
  
     
-
+    allDiseaseRelated={}
         
     for i in (Patients):       
-        patientGenes = list(expressionMetabolic.columns[expressionMetabolic.loc[i, ]==1])
+        patientGenes = list(expressionMetabolicBinary.columns[expressionMetabolicBinary.loc[i, ]==1])
         #count each gene
         for ii in patientGenes:
             if ii in allDiseaseRelated.keys():
@@ -253,7 +381,7 @@ def filteredGenesGeneBase(G, Patients, expressionMetabolic, numUpDown, f,
 
     filteredPatientGenesList=[]
 
-    if f >=0:
+    if f > 0:
         for (key, value) in allDiseaseRelated.items():
             if value>=f:
                 filteredPatientGenesList.append(key)
@@ -261,7 +389,9 @@ def filteredGenesGeneBase(G, Patients, expressionMetabolic, numUpDown, f,
         for (key, value) in allDiseaseRelated.items():
             if value<=-f:
                 filteredPatientGenesList.append(key)
-    
+    elif f == 0:
+        filteredPatientGenesList = list(allDiseaseRelated.keys())
+        
     return filteredPatientGenesList
 
 
