@@ -3,14 +3,12 @@ import numpy as np
 import networkx as nx
 import math
 import random
-#from datetime import datetime
 import os
 from scipy import stats
 
 from conncomb.randomconnectivity import randomConnectivity
 
 
-
 def patientBased(disease, network, networkPath, currencyMetaPer, 
         countDataPath, sampleDataPath, diseaseGenesPath, 
         upORdownRegulated, 
@@ -21,7 +19,7 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
         method="patientBase",
         combination = "X0"):     
     
-    #directory = datetime.today().strftime('%d%m%Y')
+    
     if not os.path.exists("result/"+directory):
         os.makedirs("result/"+directory)
     
@@ -34,6 +32,9 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
     ##### Gene Expression Data #####
     countData = pd.read_csv(countDataPath, index_col=0)
     
+    if any("ENSG" in s for s in countData.index):
+        countData = countData.T
+
     ##### Patient/Control Data #####
     sampleData = pd.read_csv(sampleDataPath, index_col=0)
     
@@ -64,6 +65,7 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
     expressionMetabolicS = countData.loc[Patients+Controls, list(GCS.nodes())]    
     expressionMetabolicS = expressionMetabolicS.apply(pd.to_numeric, errors='coerce')
     
+    fRange = [i for i in fRange if i<=numSamples]
     
 # =============================================================================
 # Connectivity    
@@ -107,6 +109,8 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
         numNodEdgBinP = np.where(numNodEdgP<minNumNodes,0,1)
         connAnalysisPatients = connAnalysisPatientsAll[0]*numNodEdgBinP
         
+        connAnalysisDiff = connAnalysisPatients-connAnalysisControl
+        
         #revise random connectivity wrt min num of nodes
         randomConnectivityControlMeanB = randomConnectivityControlMean * numNodEdgBinC
         randomConnectivityControlStdB = randomConnectivityControlStd * numNodEdgBinC
@@ -123,36 +127,133 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
         zScoreDiff = connAnalysisPatient_zscore-connAnalysisControl_zscore
      
     
+        
+        numGenes = GCS.number_of_nodes()
+  
+        
+        
         # find the index that max diff
-        indexMax = list(zScoreDiff.stack().idxmax())
+        indexMaxZ = list(zScoreDiff.stack().idxmax())
         #maxZscore = zScoreDiff.max().max()
        
     
         ##### Max Diff Filtered Genes #####
-        pG, f = indexMax
-        numGenes = GCS.number_of_nodes()
-        numUpDown = math.ceil(numGenes * pG)
+        pGZ, fZ = indexMaxZ
+
+        numUpDownZ = math.ceil(numGenes * pGZ)
+        
+        
+        PatientGenesZ_patients = filteredGenesPatientBase(GCS, Patients, expressionMetabolicS,
+                                                numUpDownZ, fZ, upORdownRegulated)
+        
+        PatientGenesZ_controls = filteredGenesPatientBase(GCS, Controls, expressionMetabolicS,
+                                                numUpDownZ, fZ, upORdownRegulated)
         
-        PatientGenes = filteredGenesPatientBase(GCS, Patients, expressionMetabolicS,
-                                                numUpDown, f, upORdownRegulated)
         #connectivity
-        SP = GCS.subgraph(PatientGenes)        
+        SPZ = GCS.subgraph(PatientGenesZ_patients)        
         
         
         ##### GWAS Match #####
-        Intersection = len(set(SP.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
-        
-        detectedDiseaseGenes = set(SP.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl)))        
-        
+        IntersectionZ = len(set(SPZ.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+        detectedDiseaseGenesZ = set(SPZ.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl)))
         diseaseGenesInGraph = len(set(GCS.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
     
+        
+# =============================================================================
+# Intersection vs Random    
+# =============================================================================
+        if SPZ.number_of_nodes() == GCS.number_of_nodes():
+                        
+            intRandomMeanZ = IntersectionZ#####################
+            intRandomStdZ = 0
+            intZscoreZ = 0
+            
+        elif SPZ.number_of_nodes() == 0:
+            intRandomMeanZ = 0
+            intRandomStdZ = 0
+            intZscoreZ = 0       
+            
+        else:
+            numDetGeneZ = SPZ.number_of_nodes()
+            
+            intersectionSampleSizeZ=[]
+            for i in range(numRandomGroups):
+                sampZ = random.sample(list(expressionMetabolicS.columns), numDetGeneZ)
+                intersectionSampleZ = len(set(diseaseGenesListEnsembl.ensembl).intersection(set(sampZ)))
+                intersectionSampleSizeZ.append(intersectionSampleZ)
+                
+        
+            intRandomMeanZ = np.mean(intersectionSampleSizeZ)
+            intRandomStdZ = np.std(intersectionSampleSizeZ)
+            intZscoreZ = (IntersectionZ-intRandomMeanZ)/intRandomStdZ########################
+       
+# =============================================================================
+#     
+# =============================================================================
+        
+        # find the index that max diff
+        indexMaxC = list(connAnalysisDiff.stack().idxmax())
+        #maxZscore = zScoreDiff.max().max()
+       
     
-        ##### SAVE #####
+        ##### Max Diff Filtered Genes #####
+        pGC, fC = indexMaxC
+
+        numUpDownC = math.ceil(numGenes * pGC)
+        
+        
+        PatientGenesC_patients = filteredGenesPatientBase(GCS, Patients, expressionMetabolicS,
+                                                numUpDownC, fC, upORdownRegulated)
+        
+        PatientGenesC_controls = filteredGenesPatientBase(GCS, Controls, expressionMetabolicS,
+                                                numUpDownC, fC, upORdownRegulated)
+        
+        #connectivity
+        SPC = GCS.subgraph(PatientGenesC_patients)        
+        
+        
+        ##### GWAS Match #####
+        IntersectionC = len(set(SPC.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+        detectedDiseaseGenesC = set(SPC.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl)))
+        #diseaseGenesInGraph = len(set(GCS.nodes()).intersection(set(list(diseaseGenesListEnsembl.ensembl))))
+    
+ 
+        if SPC.number_of_nodes() == GCS.number_of_nodes():
+                        
+            intRandomMeanC = IntersectionC#####################
+            intRandomStdC = 0
+            intZscoreC = 0
+            
+        elif SPC.number_of_nodes() == 0:
+            intRandomMeanC = 0
+            intRandomStdC = 0
+            intZscoreC = 0       
+            
+        else:
+            numDetGeneC = SPC.number_of_nodes()
+            
+            intersectionSampleSizeC=[]
+            for i in range(numRandomGroups):
+                sampC = random.sample(list(expressionMetabolicS.columns), numDetGeneC)
+                intersectionSampleC = len(set(diseaseGenesListEnsembl.ensembl).intersection(set(sampC)))
+                intersectionSampleSizeC.append(intersectionSampleC)
+                
+        
+            intRandomMeanC = np.mean(intersectionSampleSizeC)
+            intRandomStdC = np.std(intersectionSampleSizeC)
+            intZscoreC = (IntersectionC-intRandomMeanC)/intRandomStdC########################
+       
+    
+    
+# =============================================================================
+#  Save       
+# =============================================================================
         
         resultDict = {"disease":disease, "network": network,
                     "currencyMetaPer": currencyMetaPer,
                     "upORdownRegulated": upORdownRegulated,
                     "method": method,
+                    "combination": combination,
                     "minNumberNodes": minNumNodes, #eliminate graphs that has less than "minNumberNodes" nodes
                     "numRandomGroups":numRandomGroups,#number of randomly generated groups to estimate avg, std connectivity
                     "numNodes":GCS.number_of_nodes(),
@@ -161,37 +262,62 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
                     
                     "numDiseaseGenes":len(diseaseGenesListEnsembl),
                     "numDiseaseGenesInGraph":diseaseGenesInGraph,
-                    "numDetectedDiseaseGenes":SP.number_of_nodes(),
                     
-                    "subgraphEdges":SP.number_of_edges(),
-                    "maxDiffIndex_pG":indexMax[0],
-                    "maxDiffIndex_f":indexMax[1],
+                    "numDetectedDiseaseGenesBYZ":SPZ.number_of_nodes(),
                     
-                    "maxDiffPatientConn":connAnalysisPatients.loc[pG, f],
-                    "maxDiffControlConn":connAnalysisControl.loc[pG, f],
-                    "maxDiffConnDiff":connAnalysisPatients.loc[pG, f]-connAnalysisControl.loc[pG, f],
-                    "maxDiffPatientZScore":connAnalysisPatient_zscore.loc[pG, f],
-                    "maxDiffControlZScore":connAnalysisControl_zscore.loc[pG, f],
-                    "maxDiffZScoreDiff":connAnalysisPatient_zscore.loc[pG, f]-connAnalysisControl_zscore.loc[pG, f],
+                    "subgraphEdgesBYZ":SPZ.number_of_edges(),
+                    "maxDiffIndex_pG_BYZ":indexMaxZ[0],
+                    "maxDiffIndex_f_BYZ":indexMaxZ[1],
                     
-                    "intersection":Intersection
+                    "maxDiffPatientConn_BYZ":connAnalysisPatients.loc[pGZ, fZ],
+                    "maxDiffControlConn_BYZ":connAnalysisControl.loc[pGZ, fZ],
+                    "maxDiffConnDiff_BYZ":connAnalysisPatients.loc[pGZ, fZ]-connAnalysisControl.loc[pGZ, fZ],
+                    "maxDiffPatientZScore_BYZ":connAnalysisPatient_zscore.loc[pGZ, fZ],
+                    "maxDiffControlZScore_BYZ":connAnalysisControl_zscore.loc[pGZ, fZ],
+                    "maxDiffZScoreDiff_BYZ":connAnalysisPatient_zscore.loc[pGZ, fZ]-connAnalysisControl_zscore.loc[pGZ, fZ],
+                    
+                    "intersection_BYZ":IntersectionZ,
+                    
+                    "randomIntersectionNumNodesMean_BYZ": intRandomMeanZ,
+                    "randomIntersectionNumNodesStd_BYZ": intRandomStdZ,
+                    "randomIntersectionNumNodesZScore_BYZ": intZscoreZ,
+                    
+                    
+                    
+                    "numDetectedDiseaseGenesBYC":SPC.number_of_nodes(),
+                    
+                    "subgraphEdgesBYC":SPC.number_of_edges(),
+                    "maxDiffIndex_pG_BYC":indexMaxC[0],
+                    "maxDiffIndex_f_BYC":indexMaxC[1],
+                    
+                    "maxDiffPatientConn_BYC":connAnalysisPatients.loc[pGC, fC],
+                    "maxDiffControlConn_BYC":connAnalysisControl.loc[pGC, fC],
+                    "maxDiffConnDiff_BYC":connAnalysisPatients.loc[pGC, fC]-connAnalysisControl.loc[pGC, fC],
+                    
+                    
+                    "intersection_BYC":IntersectionC,
+                    
+                    "randomIntersectionNumNodesMean_BYC": intRandomMeanC,
+                    "randomIntersectionNumNodesStd_BYC": intRandomStdC,
+                    "randomIntersectionNumNodesZScore_BYC": intZscoreC
                     }
         
         resultDF = pd.DataFrame.from_dict(resultDict, orient="index")
         resultDF0 = pd.concat([resultDF0, resultDF.T])
-        
-# =============================================================================
-# Save         
-# =============================================================================
-      
+              
         
         fn1="result/" + directory + "/"
-        fn2= "_" +method + "_" + combination + "_" + disease +"_"+ network + "_" + upORdownRegulated + "_currMet" + str(currencyMetaPer*10).replace(".", "") +"_minNode"+str(minNumNodes)+ ".csv"
+        fn2= "_" +method + "_" + combination + "_" + disease +"_"+ network + "_" + upORdownRegulated + "_currMet" + str(currencyMetaPer).replace(".", "") +"_minNode"+str(minNumNodes)+ ".csv"
         
         resultDF0.to_csv(fn1+"resultSummary"+fn2)
         
-        pd.DataFrame(PatientGenes).to_csv(fn1+"patientGeneList"+fn2)
-        pd.DataFrame(detectedDiseaseGenes).to_csv(fn1+"detectedDiseaseGenes"+fn2)
+        pd.DataFrame(PatientGenesZ_patients).to_csv(fn1+"patientGeneListBYZ"+fn2)
+        pd.DataFrame(PatientGenesZ_controls).to_csv(fn1+"controlGeneListBYZ"+fn2)
+        pd.DataFrame(detectedDiseaseGenesZ).to_csv(fn1+"detectedDiseaseGenesBYZ"+fn2)
+        
+        pd.DataFrame(PatientGenesC_patients).to_csv(fn1+"patientGeneListBYZ"+fn2)
+        pd.DataFrame(PatientGenesC_controls).to_csv(fn1+"controlGeneListBYZ"+fn2)
+        pd.DataFrame(detectedDiseaseGenesC).to_csv(fn1+"detectedDiseaseGenesBYZ"+fn2)
         
         numNodEdgC.to_csv(fn1+"numberNodesControl"+fn2)
         numNodEdgP.to_csv(fn1+"numberNodesPatient"+fn2)
@@ -215,6 +341,56 @@ def patientBased(disease, network, networkPath, currencyMetaPer,
 
 
 
+def filteredGenesPatientBase(G, Patients, expressionMetabolic, numUpDown, f, upORdownRegulated):
+    
+    """
+    G: gene centric metabolic network
+    Patients: patients list
+    expressionMetabolic: expression matrix
+    numUpDown: number of up/down regulated genes
+    f: filtering threshold
+    """
+    
+    allDiseaseRelated={}
+    #connectivityPatients=[]
+
+    for i in (Patients):
+        
+        if upORdownRegulated == "updown":        
+            z = np.abs(stats.zscore(expressionMetabolic.loc[i]))
+            z = pd.Series(z)
+            patientGenes = list(z.sort_values(ascending=False).head(numUpDown).index)
+        elif upORdownRegulated == "up":
+            z = stats.zscore(expressionMetabolic.loc[i])
+            z = pd.Series(z)
+            patientGenes = list(z.sort_values(ascending=False).head(numUpDown).index)           
+        elif upORdownRegulated == "down":
+            z = stats.zscore(expressionMetabolic.loc[i])
+            z = pd.Series(z)
+            patientGenes = list(z.sort_values(ascending=False).tail(numUpDown).index)
+
+        #count each gene
+        for ii in patientGenes:
+            if ii in allDiseaseRelated.keys():
+                allDiseaseRelated[ii]=allDiseaseRelated[ii]+1
+            else:
+                allDiseaseRelated[ii]=1
+
+    filteredPatientGenesList=[]
+
+    if f > 0:
+        for (key, value) in allDiseaseRelated.items():
+            if value>=f:
+                filteredPatientGenesList.append(key)
+    elif f < 0:
+        for (key, value) in allDiseaseRelated.items():
+            if value<=-f:
+                filteredPatientGenesList.append(key)
+    elif f == 0:
+        filteredPatientGenesList = list(allDiseaseRelated.keys())
+        
+    return filteredPatientGenesList
+
 def connectivityPatientBase(G, expressionMetabolic, Patients, 
                             pGRange, fRange, upORdownRegulated="updown"):  
     
@@ -256,50 +432,3 @@ def connectivityPatientBase(G, expressionMetabolic, Patients,
             
     return connP, numNodEdg
 
-def filteredGenesPatientBase(G, Patients, expressionMetabolic, numUpDown, f, upORdownRegulated):
-    
-    """
-    G: gene centric metabolic network
-    Patients: patients list
-    expressionMetabolic: expression matrix
-    numUpDown: number of up/down regulated genes
-    f: filtering threshold
-    """
-    
-    allDiseaseRelated={}
-    #connectivityPatients=[]
-
-    for i in (Patients):
-        
-        if upORdownRegulated == "updown":        
-            z = np.abs(stats.zscore(expressionMetabolic.loc[i]))
-            z = pd.Series(z)
-            patientGenes = list(z.sort_values(ascending=False).head(numUpDown).index)
-        elif upORdownRegulated == "up":
-            z = stats.zscore(expressionMetabolic.loc[i])
-            z = pd.Series(z)
-            patientGenes = list(z.sort_values(ascending=False).head(numUpDown).index)           
-        elif upORdownRegulated == "down":
-            z = stats.zscore(expressionMetabolic.loc[i])
-            z = pd.Series(z)
-            patientGenes = list(z.sort_values(ascending=False).tail(numUpDown).index)
-
-        #count each gene
-        for ii in patientGenes:
-            if ii in allDiseaseRelated.keys():
-                allDiseaseRelated[ii]=allDiseaseRelated[ii]+1
-            else:
-                allDiseaseRelated[ii]=1
-
-    filteredPatientGenesList=[]
-
-    if f >=0:
-        for (key, value) in allDiseaseRelated.items():
-            if value>=f:
-                filteredPatientGenesList.append(key)
-    elif f < 0:
-        for (key, value) in allDiseaseRelated.items():
-            if value<=-f:
-                filteredPatientGenesList.append(key)
-    
-    return filteredPatientGenesList
\ No newline at end of file
